@@ -33,20 +33,21 @@ import threading
 import asyncio
 
 from asyncio.base_events import Server
+from concurrent.futures import ThreadPoolExecutor
 from asyncio.streams import StreamReader, StreamWriter
 from ssl import PROTOCOL_TLS_SERVER, SSLContext
 from collections import OrderedDict
 from socketserver import ThreadingMixIn, TCPServer
 from time import sleep
-from urllib.parse import unquote, quote
+from urllib.parse import unquote
 
-from typing import Callable, Dict, List, Tuple
+from typing import Any, Callable, Dict, List, Tuple
 
 from simple_http_server import ControllerFunction, StaticFile
 from .http_protocol_handler import HttpProtocolHandler, SocketServerStreamRequestHandlerWraper
 from .wsgi_request_handler import WSGIRequestHandler
 
-from .__utils import remove_url_first_slash, get_function_args, get_function_kwargs
+from .__utils import remove_url_first_slash, get_function_args, get_function_kwargs, get_path_reg_pattern
 from .logger import get_logger
 
 _logger = get_logger("simple_http_server.http_server")
@@ -111,43 +112,6 @@ class RoutingConf:
             self._res_conf.append((key, val))
         self._res_conf.sort(key=lambda it: -len(it[0]))
 
-    def __get_path_reg_pattern(self, url):
-        _url: str = url
-        path_names = re.findall("(?u)\\{\\w+\\}", _url)
-        if len(path_names) == 0:
-            if _url.startswith("**"):
-                _url = _url[2: ]
-                assert _url.find("*") < 0, "You can only config a * or ** at the start or end of a path."
-                _url = f'^([\\w%.-@!\\(\\)\\[\\]\\|\\$/]+){_url}$'
-                return _url, [quote("__path_wildcard")]
-            elif _url.startswith("*"):
-                _url = _url[1: ]
-                assert _url.find("*") < 0, "You can only config a * or ** at the start or end of a path."
-                _url = f'^([\\w%.-@!\\(\\)\\[\\]\\|\\$]+){_url}$'
-                return _url, [quote("__path_wildcard")]
-            elif _url.endswith("**"):
-                _url = _url[0: -2]
-                assert _url.find("*") < 0, "You can only config a * or ** at the start or end of a path."
-                _url = f'^{_url}([\\w%.-@!\\(\\)\\[\\]\\|\\$/]+)$'
-                return _url, [quote("__path_wildcard")]
-            elif _url.endswith("*"):
-                _url = _url[0: -1]
-                assert _url.find("*") < 0, "You can only config a * or ** at the start or end of a path."
-                _url = f'^{_url}([\\w%.-@!\\(\\)\\[\\]\\|\\$]+)$'
-                return _url, [quote("__path_wildcard")]
-            else:
-                # normal url
-                return None, path_names
-        for name in path_names:
-            _url = _url.replace(name, "([\\w%.-@!\\(\\)\\[\\]\\|\\$]+)")
-        _url = f"^{_url}$"
-
-        quoted_names = []
-        for name in path_names:
-            name = name[1: -1]
-            quoted_names.append(quote(name))
-        return _url, quoted_names
-
     def map_controller(self, ctrl: ControllerFunction):
         url = ctrl.url
         regexp = ctrl.regexp
@@ -161,7 +125,7 @@ class RoutingConf:
         else:
             _url = remove_url_first_slash(url)
 
-            path_pattern, path_names = self.__get_path_reg_pattern(_url)
+            path_pattern, path_names = get_path_reg_pattern(_url)
             if path_pattern is None:
                 self.method_url_mapping[_method][_url] = ctrl
             else:
@@ -249,19 +213,33 @@ class RoutingConf:
                 return fun, path_values
         return None
 
-    def map_filter(self, path_pattern, filter_fun):
-        self.filter_mapping[path_pattern] = filter_fun
+    def map_filter(self, filter_conf: Dict[str, Any]):
+        # {"path": p, "url_pattern": r, "func": filter_fun}
+        path = filter_conf["path"] if "path" in filter_conf else ""
+        regexp = filter_conf["url_pattern"]
+        filter_fun = filter_conf["func"]
+        if path:
+            regexp = get_path_reg_pattern(path)[0]
+            if not regexp:
+                regexp = f"^{path}$"
+        _logger.debug(f"[path: {path}] map url regexp {regexp} to function: {filter_fun}")
+        self.filter_mapping[regexp] = filter_fun
 
     def get_matched_filters(self, path):
+        return self._get_matched_filters(remove_url_first_slash(path)) + self._get_matched_filters(path)
+
+    def _get_matched_filters(self, path):
         available_filters = []
-        for key, val in self.filter_mapping.items():
-            if re.match(key, path):
+        for regexp, val in self.filter_mapping.items():
+            m = re.match(regexp, path)
+            _logger.debug(f"filter:: [{regexp}], path:: [{path}] match? {m is not None}")
+            if m:
                 available_filters.append(val)
         return available_filters
 
     def map_websocket_handler(self, endpoint, handler_class):
         url = remove_url_first_slash(endpoint)
-        path_pattern, path_names = self.__get_path_reg_pattern(url)
+        path_pattern, path_names = get_path_reg_pattern(url)
         if path_pattern is None:
             self.ws_mapping[url] = handler_class
         else:
@@ -377,6 +355,8 @@ class HTTPServer(TCPServer, RoutingConf):
 
     allow_reuse_address = 1    # Seems to make sense in testing environment
 
+    _default_max_workers = 50
+
     def server_bind(self):
         """Override server_bind to store the server name."""
         TCPServer.server_bind(self)
@@ -384,12 +364,29 @@ class HTTPServer(TCPServer, RoutingConf):
         self.server_name = socket.getfqdn(host)
         self.server_port = port
 
-    def __init__(self, addr, res_conf={}):
+    def __init__(self, addr, res_conf={}, max_workers: int = None):
         TCPServer.__init__(self, addr, SocketServerStreamRequestHandlerWraper)
         RoutingConf.__init__(self, res_conf)
+        self.max_workers = max_workers or self._default_max_workers
+        self.threadpool: ThreadPoolExecutor = ThreadPoolExecutor(
+            thread_name_prefix="ReqThread",
+            max_workers=self.max_workers)
 
+    def process_request_thread(self, request, client_address):
+        try:
+            self.finish_request(request, client_address)
+        except Exception:
+            self.handle_error(request, client_address)
+        finally:
+            self.shutdown_request(request)
 
-class ThreadingMixInHTTPServer(ThreadingMixIn, HTTPServer):
+    # override
+    def process_request(self, request, client_address):
+        self.threadpool.submit(self.process_request_thread, request, client_address)
+
+    def server_close(self):
+        super().server_close()
+        self.threadpool.shutdown(True)
 
     def start(self):
         self.serve_forever()
@@ -452,8 +449,8 @@ class CoroutineHTTPServer(RoutingConf):
 class SimpleDispatcherHttpServer:
     """Dispatcher Http server"""
 
-    def map_filter(self, path_pattern, filter_fun):
-        self.server.map_filter(path_pattern, filter_fun)
+    def map_filter(self, filter_conf):
+        self.server.map_filter(filter_conf)
 
     def map_controller(self, ctrl: ControllerFunction):
         self.server.map_controller(ctrl)
@@ -474,7 +471,8 @@ class SimpleDispatcherHttpServer:
                  keypass: str = "",
                  ssl_context: SSLContext = None,
                  resources: Dict[str, str] = {},
-                 prefer_corountine=False):
+                 prefer_corountine=False,
+                 max_workers: int = None):
         self.host = host
         self.__ready = False
 
@@ -499,12 +497,12 @@ class SimpleDispatcherHttpServer:
                 self.host[0], self.host[1], self.ssl_ctx, resources)
         else:
             _logger.info(f"Start server in threading mixed mode, listen to port {self.host[1]}")
-            self.server = ThreadingMixInHTTPServer(self.host, resources)
+            self.server = HTTPServer(self.host, resources, max_workers=max_workers)
             if self.ssl_ctx:
                 self.server.socket = self.ssl_ctx.wrap_socket(
                     self.server.socket, server_side=True)
 
-    @ property
+    @property
     def ready(self):
         return self.__ready
 
